/*********************************************************************************
 * Includes
 ********************************************************************************/
// Basically, first, as on top of all HAM files, the most
// important include is the mygba.h, which contains all the
// macro definitions, and the HAMlib function prototypes.
#include <hel.h>
// Graphics
#include "gfx/master.pal.c"
#include "gfx/balls.raw.c"
#include "gfx/bg.raw.c"
#include "gfx/bg.pal.c"
// Lookup tables for the sprite wobbling
#include "lookups.h"


/*********************************************************************************
 * Defines
 ********************************************************************************/
#define NUM_SPRITES 40  // You can set this to max 127


/*********************************************************************************
 * Global Variables
 ********************************************************************************/
u8 g_NewFrame=TRUE;
u32 g_Frames=0;


/*********************************************************************************
 * Prototypes
 ********************************************************************************/
void vblFunc(void);


/*********************************************************************************
 * Program entry point
 ********************************************************************************/
int main(void)
{
    u8 i;
    u8 arSprite[NUM_SPRITES];

    // Initialize HAMlib
    ham_Init();
    ham_SetBgMode(4);

    // load backgroundpicture
    ham_LoadBitmap((void*)&bg_Bitmap);
    ham_FlipBGBuffer();

   // create an object
   arSprite[0] = ham_CreateObj((void *)&balls_Bitmap[0],  // A pointer to the tile data for this object
                              0,                       // obj_shape
                              2,                       // obj_size
      /* See here ------> */  OBJ_MODE_OBJWINDOW,      // obj_mode
                              1,                       // col_mode
                              0,                       // pal_no
                              0,                       // mosaic
                              0,                       // hflip
                              0,                       // vflip
                              0,                       // dbl_size
                              0,                       // prio
                              30,                      // x position of sprite on screen
                              30);                     // y position of sprite on screen
                              
    // Clone object
    for(i=1; i<NUM_SPRITES; i++)
    {
        // Cloning a sprite means having another ham_obj entry that you can play
        // with that uses a copy of the data set up for the sprite specified in objno.
    	 arSprite[i] = ham_CloneObj(arSprite[0], 1, 1);
    }

    // Setup an OBJ-Window
    // Since only transparent obj-pixels form the window,
    // we do not need to specify any region
    hel_WinInit(2, 0, 0, 0, 0, WIN_NONE, WIN_ALL);

    // Load Background Palette
    ham_LoadBGPal256((void *)&bg_Palette);
    
    // Inits the Palette for the Sprite
    ham_LoadObjPal256((void *)&master_Palette); 
    
   // Will, once activated, trigger the specified interrupt,
   // and then call the function specified
   ham_StartIntHandler(INT_TYPE_VBL,         // The Interrupts ID you want to start.
                       (void *)&vblFunc);    // The adress of a function that should be called when the interrupt is fired

    // Loop
    while(1)
    {
        // New Frame?
        if(g_NewFrame)
        {
            // update the sprite positions.
            for(i=0;i<NUM_SPRITES;i++)
            {
                // Sets the X and Y Position of the Sprite number specified
                hel_ObjSetXY(arSprite[i], wobble_x[(g_Frames+(i*2))%120], wobble_y[(g_Frames+(i*3))%300]);
            }
            
            g_NewFrame=FALSE;
        }
    }
}

/*********************************************************************************
 * vblFunc(void)
 *
 * This function is called whenever the GBA is about
 * to draw a new picture onto the screen.
 ********************************************************************************/
void vblFunc()
{
   // Call this (preferably during a VBL interrupt) to commit your
   // ham_obj information to the hardware. Only after you did this your
   // sprite changes will appear on screen.
   ham_CopyObjToOAM();
              	
	//signify new frame
	g_NewFrame=TRUE;
	g_Frames++;
}

/* END OF FILE */
