/*
 * This document has been created by using Visual HAM
 * http://www.console-dev.de
 *
 * Author: Peter Schraut
 *
 * This example shows how to use the Custom-TextDraw-System with
 * a 256 color bitmap in mode 1, using a rotation map !
*/
/*********************************************************************************
 * Includes
 ********************************************************************************/
// Basically, first, as on top of all HAM files, the most
// important include is the mygba.h, which contains all the
// macro definitions, and the HAMlib function prototypes.
#include <mygba.h>
#include <hel.h>

#include "gfx/Charset.raw.c"
#include "gfx/Charset.pal.c"

/*********************************************************************************
 * Globals
 ********************************************************************************/
u8 g_NewFrame=TRUE;
u16 g_Angle=10;

/*
    Setup a "custom_text_info" structure.
*/
TCustomTextInfo g_TextInfo=
{
    // background to use
    2,
    // order of chars on bitmap.
    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789?,.-;: #^+*@!\"%\%&/()=?|\\<>[]{}"
};

/*********************************************************************************
 * Prototypes
 ********************************************************************************/
void vblFunc(void);


/*********************************************************************************
 * Program entry point
 ********************************************************************************/
int main(void)
{
    // Initialize HAMlib
    // Then we call the single most important function of ham, ham_Init .
    // This function must be called first thing when working with HAMlib,
    // because all ham_* functions will only work correctly after this has been called.
    // Behaviour of HAMlib is undefined if you comment this out.
    ham_Init();

    // Set to mode 1
    ham_SetBgMode(1);
    
    // Load 256 color palette
    ham_LoadBGPal256((void*)Charset_Palette);
    
    // Initialize the custom textdraw system
    hel_CustomTextInit(&g_TextInfo,                 // Pointer to custom_text_info
                       (void*)Charset_Tiles,        // Pointer to the tile source data
                       SIZEOF_16BIT(Charset_Tiles), // Size of the tiles to be copied into Tile RAM, in number of 16bit chunks
                       1,                           // Color mode of the tile set: 0 = 16colors/16 palettes 1 = 256 colors / 1 palette
                       0,                           // PaletteNumber, just specify 0 when using 256 colors
                       1,                           // The size of the map (0-3). This depends on the map_rot parameter!
                       1);                          // 0 = normal map, 1 = rotation map
                 //   ^^^ Notice the 1 here, for rotation map!!!


    // Turn on the background
    ham_InitBg(g_TextInfo.BgNo, BG_ACTIVE_ON, BG_PRIO_1, BG_MOSAIC_OFF);

    // Output a string on a tiles mode GBA screen.
    hel_CustomTextPrint(&g_TextInfo,                        // Pointer to custom_text_info structure
                        0,                                  // X character number the string should start on
                        1,                                  // Y character number the string should start on
                        "Custom-Textdraw-System Example");  // The string to output
    
    // Print moooore text :)
    hel_CustomTextPrint(&g_TextInfo, 8, 6, "O  O OOO O  ");
    hel_CustomTextPrint(&g_TextInfo, 8, 7, "O  O O   O  ");
    hel_CustomTextPrint(&g_TextInfo, 8, 8, "OOOO OOO O  ");
    hel_CustomTextPrint(&g_TextInfo, 8, 9, "O  O O   O  ");
    hel_CustomTextPrint(&g_TextInfo, 8,10, "O  O OOO OOO");
    hel_CustomTextPrint(&g_TextInfo, 2,14, "Press LEFT, RIGHT to rotate");
    hel_CustomTextPrint(&g_TextInfo, 7,18, "HEL rocks, fact!");
    
    // Will, once activated, trigger the specified interrupt,
    // and then call the function specified
    ham_StartIntHandler(INT_TYPE_VBL,         // The Interrupts ID you want to start.
                        (void *)&vblFunc);    // The adress of a function that should be called when the interrupt is fired

    // Loop
    while(TRUE)
    {
        // It's a new frame?
        if(g_NewFrame)
        {
            // Query joypad
            if(F_CTRLINPUT_LEFT_PRESSED) if(--g_Angle < 1) g_Angle=359;
            if(F_CTRLINPUT_RIGHT_PRESSED) if(++g_Angle > 359) g_Angle=0;
            
            // Rotate background
            ham_RotBg(g_TextInfo.BgNo, g_Angle%360, 120, 80, 256);
            
            // Frames isn't new anymore
            g_NewFrame=FALSE;
        }
    }

    return 0;
}

/*********************************************************************************
 * vblFunc(void)
 *
 * This function is called whenever the GBA is about
 * to draw a new picture onto the screen.
 ********************************************************************************/
void vblFunc(void)
{
    // Call this (preferably during a VBL interrupt) to commit your
    // ham_obj information to the hardware. Only after you did this your
    // sprite changes will appear on screen.
    ham_CopyObjToOAM();

    // It's a new frame yet
    g_NewFrame=TRUE;
}
/* END OF FILE */
