/*
 * This demo shows how to use the HEL's map-system for parallax scrolling!
 *
 * Map & graphics borrowed from Nessie's MapEd sample. (http://nessie.gbadev.org/)
 *
 * The collision in this example does not work 100%, I'm aware of it! However,
 * the used implementation in HEL is ok, it's just my lame example which is awful :/
 *
*/
/*********************************************************************************
 * Includes
 ********************************************************************************/
// Basically, first, as on top of all HAM files, the most
// important include is the mygba.h, which contains all the
// macro definitions, and the HAMlib function prototypes.
#include <mygba.h>
// include the HEL library header
#include <hel.h>

#include "level/BG_Externs.h"
#include "level/Hero.raw.c"
#include "level/obj.pal.c"

typedef struct
{
   u8 ObjNo;
   s16 X,Y;
   u8 W,H;
}THero;

/*********************************************************************************
 * Globals
 ********************************************************************************/
#define RATIO_SKY             PARALLAX_FLOAT_TO_RATIO(0.2f)
#define RATIO_PLATFORM        PARALLAX_FLOAT_TO_RATIO(1.3f)
#define RATIO_PLATFORM_FAR    PARALLAX_FLOAT_TO_RATIO(0.65f)
#define MAPSCROLL_STEPS       1


#define LAYER_COLLISION 3

u8 g_NewFrame=TRUE;
TMapScrollInfo g_MapInfos[4];
THero g_Hero;

/*********************************************************************************
 * Prototypes
 ********************************************************************************/
void vblFunc(void);
void doHero();

/*********************************************************************************
 * Program entry point
 ********************************************************************************/
int main(void)
{
    // Initialize HAMlib
    // Then we call the single most important function of ham, ham_Init .
    // This function must be called first thing when working with HAMlib,
    // because all ham_* functions will only work correctly after this has been called.
    // Behaviour of HAMlib is undefined if you comment this out.
    ham_Init();

    // set background mode
    ham_SetBgMode(0);

    // Will, once activated, trigger the specified interrupt,
    // and then call the function specified
    ham_StartIntHandler(INT_TYPE_VBL, (void *)&vblFunc);

    // store background palette
    ham_LoadBGPal(&sample_Palette, 256);
    ham_LoadObjPal(&obj_Palette, 256);

    // Init tilesets
    ham_bg[0].ti = ham_InitTileSet((void*)sample_Tiles, SIZEOF_16BIT(sample_Tiles),0,1);
    ham_bg[1].ti = ham_bg[0].ti;
    ham_bg[2].ti = ham_bg[0].ti;

    // Init all maps
    g_MapInfos[0] = hel_MapInit(0,  40, 22, false, (void*)Layer0_Map);
    g_MapInfos[1] = hel_MapInit(1,  79, 26, false, (void*)Layer1_Map);
    g_MapInfos[2] = hel_MapInit(2, 128, 32, false, (void*)Layer2_Map);

    // Init a Virtual-Map. This is an invisible map which just contains data, but nothing
    // that is displayed. In this situation it represents the collision-data!
    g_MapInfos[3] = hel_MapInitVirtual(128, 32, sizeof(u16), (void*)sample_Collision);
    
    // Set the parallax ratios!
    hel_MapSetParallaxRatio(&g_MapInfos[0], RATIO_SKY, RATIO_SKY);
    hel_MapSetParallaxRatio(&g_MapInfos[1], RATIO_PLATFORM_FAR, RATIO_PLATFORM_FAR);
    hel_MapSetParallaxRatio(&g_MapInfos[2], RATIO_PLATFORM, RATIO_PLATFORM);
    hel_MapSetParallaxRatio(&g_MapInfos[3], RATIO_PLATFORM, RATIO_PLATFORM);

    g_Hero.X = 20;
    g_Hero.Y = 5;
    g_Hero.W = 16;
    g_Hero.H = 16;
    g_Hero.ObjNo = hel_ObjCreate256((void*)&Hero_Bitmap[0], OBJ_SIZE_16X16, OBJ_MODE_NORMAL, g_Hero.X, g_Hero.Y);

    // turn on backgrounds
    ham_InitBg(0,1,2,0);
    ham_InitBg(1,1,1,0);
    ham_InitBg(2,1,0,0);


    // Loop
    while(1)
    {
        if(g_NewFrame)
        {
            TPoint32 ScrollValue={0,0};

            // read current joypad state
            hel_PadReadState();

            doHero();
            
            g_NewFrame=FALSE;
        }
    }

    return 0;
}


void doHero()
{
   u16 Edge[4];
   u16 Below[2];
   TPoint32 ScrollValue={0,0};
   
   Below[0] = *(u16*)hel_MapGetTilePtrAt(&g_MapInfos[3], g_Hero.X-1, g_Hero.Y+g_Hero.H+3);           // BottomLeft
   Below[1] = *(u16*)hel_MapGetTilePtrAt(&g_MapInfos[3], g_Hero.X+g_Hero.W, g_Hero.Y+g_Hero.H+3);  // BottomRight
   

   
   // When the Hero does not have anything under
   // his feets, let him fall ...
   if(Below[0] == 0 && Below[1] == 0)
   {
      g_Hero.Y+=2;
      ScrollValue.Y+=1;
   }
   else
   {
      if(hel_Pad.Held.Right)
      {
          Edge[3] = *(u16*)hel_MapGetTilePtrAt(&g_MapInfos[3], g_Hero.X+g_Hero.W, g_Hero.Y+g_Hero.H);
          
          switch(Edge[3])
          {
             case 0x0000:
             case 0x1004:
                g_Hero.X++;
                break;

             //  "/"
             case 0x3001:
             case 0x3004:
                g_Hero.X++;
                g_Hero.Y--;
                break;
                
             //  "\"
             case 0x5002:
             case 0x5004:
                g_Hero.X--;
                g_Hero.Y++;
                break;
          }

          if(g_Hero.X > 100) ScrollValue.X++;
      }
      else
      if(hel_Pad.Held.Left)
      {
          Edge[2] = *(u16*)hel_MapGetTilePtrAt(&g_MapInfos[3], g_Hero.X, g_Hero.Y+g_Hero.H);
          
          switch(Edge[2])
          {
             case 0x0000:
             case 0x1004:
                g_Hero.X--;
                break;
                
             //  "/"
             case 0x3001:
             case 0x3004:
                g_Hero.X--;
                g_Hero.Y++;
                break;
                
             //  "\"
             case 0x5002:
             case 0x5004:
                g_Hero.X--;
                g_Hero.Y--;
                break;
          }

          if(g_Hero.X < 100) ScrollValue.X--;
      }
   }
            
   hel_MapBatchScrollBy(g_MapInfos, ScrollValue.X, ScrollValue.Y, 4);
}


/*********************************************************************************
 * vblFunc(void)
 *
 * This function is called whenever the GBA is about
 * to draw a new picture onto the screen.
 ********************************************************************************/
void vblFunc(void)
{
    // Update X/Y position from our hero in HAM
    ham_SetObjXY(g_Hero.ObjNo, g_Hero.X, g_Hero.Y);
    
    // Call this (preferably during a VBL interrupt) to commit your
    // ham_obj information to the hardware. Only after you did this your
    // sprite changes will appear on screen.
    ham_CopyObjToOAM();

   // It's a new frame yet
    g_NewFrame=1;
}

/* END OF FILE */
