/*********************************************************************************
 * Includes
 ********************************************************************************/
// Basically, first, as on top of all HAM files, the most
// important include is the mygba.h, which contains all the
// macro definitions, and the HAMlib function prototypes.
#include <hel.h>
// Graphics
#include "gfx/bg.raw.c"
#include "gfx/bg.pal.c"


/*********************************************************************************
 * Global Variables
 ********************************************************************************/
u8 g_NewFrame=TRUE;
u32 g_Frames=0;


/*********************************************************************************
 * Prototypes
 ********************************************************************************/
void vblFunc(void);


/*********************************************************************************
 * Program entry point
 ********************************************************************************/
int main(void)
{
    // Initialize HAMlib
    ham_Init();
    ham_SetBgMode(4);

    // load backgroundpicture
    ham_LoadBitmap((void*)&bg_Bitmap);
    ham_FlipBGBuffer();

    // Setup an OBJ-Window
    // Since only transparent obj-pixels form the window,
    // we do not need to specify any region
    hel_WinInit(0,          // WinNo
                30,         // Left
                30,         // Top
                60,         // Right
                60,         // Bottom
                WIN_ALL,    // InsideFlags
                WIN_NONE);  // OutsideFlags

    // Load Background Palette
    ham_LoadBGPal256((void *)&bg_Palette);
    
   // Will, once activated, trigger the specified interrupt,
   // and then call the function specified
   ham_StartIntHandler(INT_TYPE_VBL,         // The Interrupts ID you want to start.
                       (void *)&vblFunc);    // The adress of a function that should be called when the interrupt is fired

    // Loop
    while(1)
    {
        // New Frame?
        if(g_NewFrame)
        {
            #define WINDOW_WIDTH  115
            #define WINDOW_HEIGHT 75
            
            static TPoint32 WinPos={30,30};
            static s32 DeltaX=1;
            static s32 DeltaY=1;
            
            if(((WinPos.X+=DeltaX) <= 0) || ((WinPos.X+=DeltaX) >= 240-WINDOW_WIDTH)) { WinPos.X-=DeltaX;  DeltaX*=-1; }
            if(((WinPos.Y+=DeltaY) <= 0) || ((WinPos.Y+=DeltaY) >= 160-WINDOW_HEIGHT)){ WinPos.Y-=DeltaY;  DeltaY*=-1; }
            
            hel_WinSetRegion(0, WinPos.X, WinPos.Y, WinPos.X+WINDOW_WIDTH, WinPos.Y+WINDOW_HEIGHT);
            
            g_NewFrame=FALSE;
        }
    }
}

/*********************************************************************************
 * vblFunc(void)
 *
 * This function is called whenever the GBA is about
 * to draw a new picture onto the screen.
 ********************************************************************************/
void vblFunc()
{
   // Call this (preferably during a VBL interrupt) to commit your
   // ham_obj information to the hardware. Only after you did this your
   // sprite changes will appear on screen.
   ham_CopyObjToOAM();
              	
	//signify new frame
	g_NewFrame=TRUE;
	g_Frames++;
}

/* END OF FILE */
